#pragma once

#include "PlatformImage.h"
#include "ImageData.h"

#include <memory>
#include <vector>

class Color;

class Image
{
public:
    Image(unsigned width, unsigned height, ImageData::Type dataType = ImageData::Type::UCHAR);
    ~Image();
    static std::unique_ptr<Image> Create(unsigned width, unsigned height, ImageData::Type dataType = ImageData::Type::UCHAR);

    unsigned getBytesPerRow() const;
    unsigned getWidth() const;
    unsigned getHeight() const;
    unsigned getBitDepth() const;
    unsigned getNumChannels() const;

    ImageData* getData();
    unsigned char getAsUnsignedChar(unsigned idx, unsigned jdx) const;
    PlatformImage* getPlatformImage();

    void setPixelValue(unsigned idx, unsigned jdx, const Color& color);
    void setWidth(unsigned width);
    void setHeight(unsigned height);
    void setBitDepth(unsigned bitDepth);
    void setNumChannels(unsigned numChannels);

private:
    void initialize();

    unsigned mWidth{1};
    unsigned mHeight{1};
    unsigned mBitDepth{8};
    unsigned mNumChannels{4};

    ImageData::Type mDataType;
    std::unique_ptr<ImageData> mData;
    std::unique_ptr<PlatformImage> mPlatformImage;
};
